import argparse
import git

from .utils import github_url_to_owner_repo

from autorelease import GitHubUser, ProjectOptions

class AutoreleaseParsingHelper(object):
    def __init__(self, parser=None, disable_defaults=False):
        if parser is None:
            parser = argparse.ArgumentParser()
        self.parser = parser
        self.parser.add_argument('-q', '--quiet', action='store_true')

        self.make_objects = []

    def add_github_parsing(self):
        self.parser.add_argument('-u', '--username', type=str,
                                 help='GitHub username')
        self.parser.add_argument('--token', type=str,
                                 help='authorization token')
        self.make_objects.append(GitHubUser)

    def add_project_parsing(self):
        self.parser.add_argument('--repo_owner', type=str)
        self.parser.add_argument('--repo_name', type=str)
        self.parser.add_argument('--project_name', type=str)
        self.make_objects.append(ProjectOptions)

    def add_repo_parsing(self):
        self.parser.add_argument('--repo', type=str, default='.')

    def parse_args(self, args=None):
        opts = self.parser.parse_args(args=args)
        return AutoreleaseParsedArguments(opts, self.make_objects)


class AutoreleaseParsedArguments(object):
    def __init__(self, opts, make_objects):
        self.opts = opts
        self.make_objects = make_objects

        self._upstream = None
        self._origin = None
        self._github_user = None
        self._project = None
        self._repo = None

    def __getattr__(self, name):
        return getattr(self.opts, name)

    def _remotes(self, internal):
        if internal is None and self.repo is not None:
            self.set_upstream_origin()

    @property
    def upstream(self):
        self._remotes(self._upstream)
        return self._upstream

    @property
    def origin(self):
        self._remotes(self._origin)
        return self._origin

    def set_upstream_origin(self, repo=None):
        if repo is None:
            repo = self.repo
        # input is git.Repo object
        upstream = [r for r in repo.remotes if r.name == 'upstream']
        origin = [r for r in repo.remotes if r.name == 'origin']
        if not upstream:
            upstream = origin

        if upstream == origin == []:
            raise RuntimeError("Can't guess data for this repository")

        assert len(upstream) == len(origin) == 1
        self.upstream = upstream[0]
        self.origin = origin[0]

    def guess_project(self):
        guess = {k: None
                 for k in ['repo_owner', 'repo_name', 'project_name']}
        if self.repo is not None:
            (owner, name) = github_url_to_owner_repo(self.upstream.url)
            guess = {'repo_owner': owner,
                     'repo_name': name,
                     'project_name': name}
        return guess

    def guess_github_user(self):
        guess = {k: None for k in ['username', 'token']}
        if self.repo is not None:
            (user, _) = github_url_to_owner_repo(self.origin.url)
            guess.update({'username': user})
        return guess

    @property
    def repo(self):
        if self._repo is None and hasattr(self.opts, 'repo'):
            self._repo = git.Repo(self.opts.repo)
            self._repo.remotes.origin.fetch()
        return self._repo

    @property
    def github_user(self):
        if self._github_user is None and GitHubUser in self.make_objects:
            self._github_user = self._make_object(GitHubUser,
                                                  self.guess_github_user,
                                                  opts=self.opts)
        return self._github_user

    @property
    def project(self):
        if self._project is None and ProjectOptions in self.make_objects:
            self._project = self._make_object(ProjectOptions,
                                              self.guess_project,
                                              self.opts)
        return self._project


    @staticmethod
    def _make_object(obj_cls, guesser, opts):
        # here's an abstract function....
        if opts is None:
            raise RuntimeError(
                "Can't retrieve object before parsing arguments. "
                + "Run AutoreleaseParsingHelper.parse_args() first.")
        kwargs = guesser()
        opts_dct = vars(opts)
        kwargs_opts = {k: opts_dct[k] for k in kwargs
                       if opts_dct[k] is not None}
        kwargs.update(kwargs_opts)
        kwargs = {k: v for (k, v) in kwargs.items() if v is not None}
        try:
            return obj_cls(**kwargs)
        except TypeError:
            return None
        # alternate: this is stricter and requires all values non-None
        #if any([v is None for v in kwargs.values()]):
            #return None
        #else:
            #return obj_cls(**kwargs)



#!/usr/bin/env python
import argparse
from autorelease import ReleaseNoteWriter, AutoreleaseParsingHelper

def make_parser():
    parser = argparse.ArgumentParser()
    auto_parser = AutoreleaseParsingHelper(parser)
    auto_parser.add_github_parsing()
    auto_parser.add_project_parsing()
    auto_parser.parser.add_argument("--conf", type=str)
    auto_parser.parser.add_argument("-o", "--output", type=str)
    auto_parser.parser.add_argument("--since-release", type=str)
    return auto_parser

def main():
    auto_parser = make_parser()
    opts = auto_parser.parse_args()
    writer = ReleaseNoteWriter(config=opts.conf,
                               since_release=opts.since_release,
                               project=opts.project,
                               github_user=opts.github_user)
    writer.write_release_notes(outfile=opts.output)

if __name__ == "__main__":
    main()



import yaml
import re
import os
import sys

def conda_recipe_version(recipe_file):
    with open(recipe_file) as f:
        dct = yaml.load(f.read(), Loader=yaml.FullLoader)
    return dct['package']['version']

def github_url_to_owner_repo(url):
    pattern = ".*github.com[\:\/]([^\/]+)\/(.*)\.git"
    match = re.match(pattern, url)
    return match.groups()


def import_setup(directory='.'):
    """Return the imported setup.py from the given directory"""
    if sys.version_info > (3, ):
        return _import_setup_py3(directory)
    else:
        return _import_setup_py2(directory)


def _import_setup_py2(directory):
    import imp
    return imp.load_source('setup', os.path.join(directory, 'setup.py'))


def _import_setup_py3(directory):
    import importlib.util
    spec = importlib.util.spec_from_file_location(
        'setup', os.path.join(directory, 'setup.py')
    )
    setup = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(setup)
    return setup



import click

from autorelease.scripts.vendor import vendor_actions
from autorelease.scripts.check import run_checks

@click.group()
def cli():
    pass

@cli.command()
@click.option('--conf', type=click.File('r'), default='autorelease.yml')
@click.option('--branch', default=None)
@click.option('--event', default=None)
def check(conf, branch, event):
    run_checks(conf, branch, event)

@click.group()
def vendor():
    pass

@vendor.command()
def actions():
    print("vendoring actions")
    vendor_actions(base_path='.')

cli.add_command(vendor)

if __name__ == "__main__":
    cli()



import re
import json

import requests
import git

from collections import namedtuple

ProjectOptions = namedtuple('ProjectOptions', ['repo_owner',
                                               'repo_name',
                                               'project_name'])

class GitHubUser(namedtuple('GitHubUser', ['username', 'token'])):
    @property
    def auth(self):
        return (self.username, self.token)

class GitHubRepoBase(object):
    """
    Parameters
    ----------
    project: :class:`.ProjectOptions`
    github_user: :class:`.GitHubUser`
    """
    def __init__(self, project, github_user):
        github_api_url = "https://api.github.com/"
        self.project = project
        self.repo_api_url = (github_api_url + "repos/" + project.repo_owner
                             + "/" + project.repo_name + "/")
        self.github_user = github_user

    def api_get(self, url_ending, params=None):
        return requests.get(url=self.repo_api_url + url_ending,
                            params=params,
                            auth=self.github_user.auth)

    def api_get_json_all(self, url_ending, params=None):
        # only for issues, which limit to 30 per return
        my_params = {}
        my_params.update(params)
        my_params.update({'sort': 'updated', 'direction': 'asc'})
        results = {}  # we use a dict to easily look up by number
        # actual return is list of values
        should_continue = True
        while should_continue:
            local_results_req = self.api_get(url_ending, my_params)
            local_results = local_results_req.json()
            if local_results:
                since = local_results[-1]['updated_at']
                # print(local_results[-1]['updated_at'],
                      # local_results[0]['updated_at'])
                my_params['since'] = since
            local_result_dict = {result['number']: result
                                 for result in local_results
                                 if result['number'] not in results}
            results.update(local_result_dict)
            should_continue = local_result_dict
            # print(results.keys())
        return list(results.values())


class GitHubReleaser(GitHubRepoBase):
    """
    Parameters
    ----------
    project : :class:`.ProjectOptions`
    version : str or :class:`packaging.versions.Version`
    repo : :class:`git.Repo`
    github_user : :class:`.GitHubUser`

    Attributes
    ----------
    release_target_commitish : str
    """
    def __init__(self, project, version, repo, github_user):
        super(GitHubReleaser, self).__init__(project, github_user)
        self.version = version
        # pr_re set in pr_pattern
        self._pr_pattern = None
        self.pr_re = None
        self.repo = repo

        self.pr_pattern = "Merge pull request #([0-9]+)"
        self.release_target_commitish = "stable"

    # THINGS YOU MIGHT WANT TO OVERRIDE
    @property
    def release_name(self):
        return self.project.project_name + " " + str(self.version)

    @property
    def tag_name(self):
        return "v" + str(self.version)

    def extract_release_notes(self, text):
        # TODO: make this more complicated
        return text

    # THINGS YOU'RE LESS LIKELY TO OVERRIDE
    @property
    def pr_pattern(self):
        return self._pr_pattern

    @pr_pattern.setter
    def pr_pattern(self, value):
        self._pr_pattern = value
        self.pr_re = re.compile(self._pr_pattern)

    def find_relevant_pr(self):
        # this uses the git log to find the most recent merge from PR
        # (assuming certain text in the commit log for PR merges)
        found = False
        commits = self.repo.iter_commits(self.release_target_commitish)
        commit = next(commits)
        while commit and not found:
            match = self.pr_re.match(commit.message)
            if match is not None:
                found = True
                pr_number = match.group(1)  # don't like hardcoded 1
            else:
                commit = next(commits)
        return int(pr_number)

    def get_pr_data(self, pr_number):
        pr_url = self.repo_api_url + "issues/" + str(pr_number)
        pr_data = requests.get(pr_url, auth=self.github_user.auth).json()
        return pr_data

    def generate_post_data(self, draft=False, prerelease=False):
        pr_number = self.find_relevant_pr()
        pr_data = self.get_pr_data(pr_number)
        pr_body = pr_data['body']
        release_notes = self.extract_release_notes(pr_body)
        post_data = {
            'tag_name': self.tag_name,
            'target_commitish': self.release_target_commitish,
            'name': self.release_name,
            'body': release_notes,
            'draft': draft,
            'prerelease': prerelease
        }
        return post_data

    def create_release(self, draft=False, prerelease=False):
        post_data = json.dumps(self.generate_post_data())
        post_status = requests.post(self.repo_api_url + "releases",
                                    data=post_data,
                                    auth=self.github_user.auth)



# This file vendored from Autorelease
import os
import subprocess

try:
    from configparser import ConfigParser, NoSectionError, NoOptionError
except ImportError:
    # py2
    from ConfigParser import ConfigParser, NoSectionError, NoOptionError

try:
    from ._installed_version import _installed_version
    from ._installed_version import _installed_git_hash
    from ._installed_version import _version_setup_depth
except ImportError:
    _installed_version = "Unknown"
    _installed_git_hash = "Unknown"
    _version_setup_depth = -1


def get_git_version():
    """
    Return the git hash as a string.

    Apparently someone got this from numpy's setup.py. It has since been
    modified a few times.
    """
    # Return the git revision as a string
    # copied from numpy setup.py
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        with open(os.devnull, 'w') as err_out:
            out = subprocess.Popen(cmd,
                                   stdout=subprocess.PIPE,
                                   stderr=err_out, # maybe debug later?
                                   env=env).communicate()[0]
        return out

    try:
        git_dir = os.path.dirname(os.path.realpath(__file__))
        out = _minimal_ext_cmd(['git', '-C', git_dir, 'rev-parse', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = 'Unknown'

    return GIT_REVISION

def _seek_parent_dirs_for_file(filename):
    rel_directory = None
    my_dir = os.path.dirname(os.path.abspath(__file__))
    rel_directory_arr = []
    while not rel_directory:
        expected_dir = os.path.join(*rel_directory_arr) \
                if rel_directory_arr else '.'
        expected = os.path.join(expected_dir, filename)
        if os.path.isfile(os.path.normpath(expected)):
            rel_directory = expected_dir
        else:
            rel_directory_arr.append('..')

        if len(rel_directory_arr) > len(my_dir.split(os.sep)):
            rel_directory_arr = []
            break

    return rel_directory


def _find_rel_path_for_file(depth, filename):
    rel_directory = None
    if depth == 0:
        rel_directory = '.'
    elif depth >= 1:
        rel_directory = os.sep.join(['..'] * depth)
    else:
        rel_directory = _seek_parent_dirs_for_file(filename)

    if rel_directory:
        return os.path.normpath(os.path.join(rel_directory, filename))
    else:
        return None


def get_setup_cfg(directory, filename="setup.cfg"):
    """Load the setup.cfg as a dict-of-dict.

    Parameters
    ----------
    directory : str
        directory for setup.cfg, relative to cwd; default '.'
    filename : str
        filename for setup.cfg; default 'setup.cfg'
    """
    if isinstance(directory, int):
        rel_path = _find_rel_path_for_file(directory, filename)
        start_dir = os.path.abspath(os.path.dirname(__file__))
        setup_cfg = os.path.normpath(os.path.join(start_dir, rel_path))
    else:
        setup_cfg = os.path.join(directory, filename)

    conf = None
    if os.path.exists(setup_cfg):
        conf = ConfigParser()
        conf.read(setup_cfg)

    return conf


def get_setup_version(default_version, directory, filename="setup.cfg"):
    version = default_version
    conf = get_setup_cfg(directory, filename)
    try:
        version = conf.get('metadata', 'version')
    except (NoSectionError, NoOptionError):
        pass  # version (or metadata) not defined in setup.cfg
    except AttributeError:
        pass  # no setup.cfg found (conf is None)
    return version


short_version = get_setup_version(_installed_version,
                                  directory=_version_setup_depth)
_git_version = get_git_version()
_is_repo = (_git_version != '' and _git_version != "Unknown")

if _is_repo:
    git_hash = _git_version
    full_version = short_version + "+g" + _git_version[:7]
    version = full_version
else:
    git_hash = "Unknown"
    full_version = short_version + "+g" + _installed_git_hash[:7] + '.install'
    version = short_version



import yaml

from packaging.version import Version

from autorelease.version_checks import parse_versions
from autorelease.utils import import_setup
from autorelease import DefaultCheckRunner

def checker_from_yaml_dict(release_check):
    repo_path = release_check.get('repo_path', '.')
    release_branches = release_check.get('release-branches', [])
    release_tag_format = release_check.get('release-tag', "v{BASE_VERSION}")
    versions = parse_versions(release_check['versions'])
    base_version = Version(list(versions.values())[0]).base_version
    release_tag = release_tag_format.format(BASE_VERSION=base_version)
    setup = import_setup(directory=repo_path)

    checker = DefaultCheckRunner(
        versions=versions,
        setup=setup,
        repo_path=repo_path
    )
    checker.release_branches = release_branches + [release_tag]
    return checker


def run_checks(yml, branch=None, event=None, allow_patch_skip=False):
    dct = yaml.load(yml, Loader=yaml.FullLoader)
    release_check = dct['release-check']
    checker = checker_from_yaml_dict(release_check)
    if branch is None and event is None:
        branch, event = checker.get_branch_event_from_github_env()

    tests = checker.select_tests_from_branch_event(branch, event,
                                                   allow_patch_skip)
    checker.run_as_test(tests)


if __name__ == "__main__":
    with open("autorelease.yml", 'r') as f:
        run_checks(f)




from __future__ import print_function
import sys
import os


def setup_is_release(setup, expected=True):
    """
    Returns
    -------
    bool or None :
        None if we can't tell
    """
    try:
        is_release = setup.IS_RELEASE
    except AttributeError:
        return None
    else:
        if is_release and expected:
            return ""
        elif not is_release and not expected:
            return ""
        else:
            return ("Unexpected value of setup.py IS_RELEASE. Found "
                    + str(is_release) + ".\n")


def requirements_consistency(setup, txt="requirements.txt",
                             conda_recipe=None):
    """
    Checks that the requirements defined in setup.py, requirements.txt, and
    (optionally) the conda recipe's meta.yaml file are all in agreement.
    Includes version checks (for pins) on requirements.txt and the conda
    recipe.
    """
    pass


def readme_rst_exists():
    """
    Checks that the README.rst file exists.

    This file should be read in as the long description in setup.py.
    """
    if os.path.isfile("README.rst"):
        return ""
    else:
        return "Missing README.rst file"



import pytest
from unittest import mock

import os


from autorelease.version import _find_rel_path_for_file

@pytest.mark.parametrize("depth, result", [
    (0, '.'), (1, '..'), (2, '..' + os.sep + '..'),
])
def test_find_rel_path_for_file(depth, result):
    expected = os.path.normpath(result + os.sep + 'setup.cfg')
    assert _find_rel_path_for_file(depth, 'setup.cfg') == expected

@pytest.mark.parametrize("expected", ['./setup.cfg', '../setup.cfg',
                                      '../../setup.cfg'])
def test_find_rel_path_for_file_finds_file(expected):
    with mock.patch('autorelease.version.os.path.isfile',
                    lambda x: x == expected):
        expected = os.path.normpath(expected)  # fix slashes on windows
        assert _find_rel_path_for_file(-1, 'setup.cfg') == expected

def test_find_rel_path_for_file_finds_no_file():
    with mock.patch('autorelease.version.os.path.isfile', lambda x: False):
        assert _find_rel_path_for_file(-1, 'setup.cfg') is None




from . import version
from .version_checks import VersionReleaseChecks
from .git_repo_checks import GitReleaseChecks
from .root_dir_checks import setup_is_release, readme_rst_exists

from .check_runners import CheckRunner, DefaultCheckRunner

from .github_release import (
    GitHubRepoBase, GitHubReleaser, ProjectOptions, GitHubUser
)

from .release_notes import ReleaseNoteWriter

from .script_utils import AutoreleaseParsingHelper

from .utils import conda_recipe_version

from . import scripts



#!/usr/bin/env python
from __future__ import print_function
import argparse
import logging
import re
import git

from autorelease import GitHubReleaser, GitHubUser, ProjectOptions

def make_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('--token', type=str, help='authorization token')
    parser.add_argument('--project_name', type=str)
    parser.add_argument('--version', type=str)
    parser.add_argument('--user', type=str, help='GitHub username')
    parser.add_argument('--repo', type=str, default='.')
    parser.add_argument('--repo_owner', type=str)
    parser.add_argument('--repo_name', type=str)
    parser.add_argument('--dry', action='store_true')
    parser.add_argument('-q', '--quiet', action='store_true')
    return parser

# move this to scripts/utils?
def make_logger(quiet=False):
    pass

def github_url_to_owner_repo(url):
    pattern = ".*github.com[\:\/]([^\/]+)\/(.*)"
    match = re.match(pattern, url)
    owner, name = match.groups()
    if name.endswith('.git'):
        name = name[:-4]
    return owner, name

def get_github_info(repo):
    upstream = [r for r in repo.remotes if r.name == 'upstream']
    origin = [r for r in repo.remotes if r.name == 'origin']
    if not upstream:
        upstream = origin

    if upstream == origin == []:
        raise RuntimeError("Can't guess data for this repository")

    assert len(upstream) == len(origin) == 1
    upstream = upstream[0]
    origin = origin[0]

    print(repo.remotes)
    for remote in repo.remotes:
        print(remote.url)
    print(upstream.url)
    print(origin.url)

    (owner, name) = github_url_to_owner_repo(upstream.url)
    (user, _) = github_url_to_owner_repo(origin.url)
    return owner, user, name

def main():
    parser = make_parser()
    opts = parser.parse_args()

    logger = make_logger(opts.quiet)

    print(opts)

    repo = git.Repo(opts.repo)
    repo.remotes.origin.fetch()
    owner, user, name = get_github_info(repo)

    if opts.repo_owner is not None:
        owner = opts.repo_owner
    if opts.repo_name is not None:
        name = opts.repo_name
    if opts.user is not None:
        user = opts.user

    github_user = GitHubUser(user, opts.token)
    project = ProjectOptions(owner, name, opts.project_name)
    releaser = GitHubReleaser(
        project=project,
        version=opts.version,
        repo=repo,
        github_user=github_user
    )
    
    # testing
    expected_pr = releaser.find_relevant_pr()
    print("Expected PR: " + str(expected_pr))
    print("PR issue: " + str(releaser.get_pr_data(expected_pr)))
    print("POST DATA:")
    print(releaser.generate_post_data())
    if not opts.dry:
        releaser.create_release()

if __name__ == "__main__":
    main()



# correct version.py is writted by setup.py
# this is just to give us a backup module in case we don't have this

version = "Unknown"
git_hash = "Unknown"
full_version = \
        "Unknown: Incorrect installation. Use pip or setup.py to install"



from __future__ import print_function

import re
import packaging.version as vers

from autorelease.version import get_setup_version  # reuse the vendored
from autorelease.utils import conda_recipe_version

version_getters = {
    'setup-cfg': lambda path: get_setup_version(None, path),
    'conda': conda_recipe_version,
}

default_args = {
    'setup-cfg': ['.']
}

def parse_versions(versions_list):
    versions = {}
    for version_type in versions_list:
        if isinstance(version_type, dict):
            version_type, arg = list(version_type.items())[0]
            args = [arg]
        else:
            args = default_args.get(version_type, [])

        versions[version_type] = version_getters[version_type](*args)
    return versions


class VersionReleaseChecks(object):
    def __init__(self, versions, strictness='strict'):
        self.versions = versions
        self.strictness = strictness

    @staticmethod
    def _version(version, strictness):
        if strictness == 'strict':
            return version
        elif strictness == 'loose':
            return vers.Version(version).public
        elif strictness == 'base-only':
            return vers.Version(version).base_version
        else:
            raise ValueError("Bad strictness: " + str(strictness))
        # TODO: I'm sure I can create more!

    def consistency(self, desired_version=None, include_package=False,
                    strictness=None):
        """Checks that the versions are consistent

        Parameters
        ----------
        desired_version: str
            optional; the version that all of these should match
        include_package: bool
            whether to check the special 'package' version for consistency
            (default False)
        strictness: str

        """
        keys_to_check = list(self.versions.keys())
        if not include_package and 'package' in keys_to_check:
            keys_to_check.remove('package')

        if desired_version is None:
            # if we have to guess, we trust setup.py
            try:
                desired_version = self.versions['setup.py']
            except KeyError:
                desired_version = self.versions[keys_to_check[0]]

        if strictness is None:
            strictness = self.strictness
        desired = self._version(desired_version, strictness)

        error_keys = []
        for key in keys_to_check:
            test = self._version(self.versions[key], strictness)
            if test != desired:
                error_keys += [key]

        # make the error message
        msg = ""
        for key in error_keys:
            msg += "Error: desired {d} != {v} ({k})\n".format(
                d=str(desired),
                v=str(self.versions[key]),
                k=str(key)
            )
        return msg

    @staticmethod
    def is_release(version, expected=True):
        version = str(version)
        msg = ""
        if expected is vers.Version(version).is_prerelease:
            msg += "Version " + str(version) + " is "
            if expected:
                msg += "not "
            msg += "a release version\n"

        return msg



import string
import pathlib
import pkg_resources
from packaging.version import Version
import autorelease

def vendor(resources, base_path, relative_target_dir):
    for resource in resources:
        orig_loc = pkg_resources.resource_filename('autorelease', resource)
        name = pathlib.Path(orig_loc).name
        target_dir = base_path / relative_target_dir
        target_dir.mkdir(parents=True, exist_ok=True)
        target_loc = base_path / relative_target_dir / name
        # print(f"cp {orig_loc} {target_loc}")
        with open(orig_loc, mode='r') as rfile:
            template = string.Template(rfile.read())

        version = Version(autorelease.version.version).base_version
        with open(target_loc, mode='w') as wfile:
            wfile.write(template.substitute(VERSION=version))

def vendor_actions(base_path):
    resources = ['autorelease-default-env.sh', 'autorelease-prep.yml',
                 'autorelease-gh-rel.yml', 'autorelease-deploy.yml']
    resources = ['gh_actions_stages/' + res for res in resources]
    target_dir = pathlib.Path('.github/workflows')
    vendor(resources, base_path, target_dir)




import pytest
from autorelease.scripts.bump_dev_version import *

def test_shared_parser():
    parser = shared_parser()
    opts = parser.parse_args([])
    assert opts.index == "https://test.pypi.org/pypi"
    assert opts.conf == "setup.cfg"
    # assert opts.output is None

@pytest.mark.parametrize("v_pypi, v_setup, expected", [
    ("1.0", "1.1", "1.1"),
    ("1.0.dev0", "1.0", "1.0.dev0"),
    ("1.0.rc0", "1.0", "1.0"),
])
def test_select_version(v_pypi, v_setup, expected):
    assert select_version(v_pypi, v_setup) == expected


@pytest.mark.parametrize("version_str, expected", [
    ("1.0", "1.0.dev0"), ("1.0.dev0", "1.0.dev1"),
])
def test_bump_dev_version(version_str, expected):
    assert bump_dev_version(version_str) == expected



#!/usr/bin/env python
from __future__ import print_function
import sys
import collections
import yaml
import requests
import dateutil.parser
import datetime

from .github_release import GitHubRepoBase, GitHubUser, ProjectOptions

class ReleaseNoteWriter(GitHubRepoBase):
    def __init__(self, config, since_release=None, project=None,
                 github_user=None):
        if isinstance(config, str):
            with open(config) as f:
                config = yaml.load(f.read(), Loader=yaml.SafeLoader)

        self.config = config
        self.since_release = since_release
        project = self._apply_config_key(project, 'project', ProjectOptions)
        github_user = self._apply_config_key(github_user, 'github_user',
                                             GitHubUser)

        super(ReleaseNoteWriter, self).__init__(project, github_user)
        self._latest_release_tag = None
        self._latest_release_tag_name = None
        self._latest_release_commit_date = None

    def _apply_config_key(self, var, key, obj_cls):
        # this is to make codeclimate happy
        if var is None and key in self.config.keys():
            var = obj_cls(**self.config[key])
        return var

    def filter_recent_pulls(self, pulls, since):
        # filter for date
        pulls = [p for p in pulls if p['merged_at'] > since]
        # ensure that you don't have the last PR from previous showing up
        # (there can be a second in time difference between commit and
        # merge)
        latest_tag_sha = self._latest_release_tag['commit']['sha']
        pulls = [p for p in pulls
                 if p['merge_commit_sha'] != latest_tag_sha]
        # print([p['number'] for p in pulls])
        return pulls

    def label_organized_merged_pulls(self, since=''):
        # the implementation challenge is that the return info from pulls
        # doesn't currently include info about labels -- that is contains in
        # the return info from issues (all pulls are issues).

        # because of problems that the pull close date doesn't always match
        # the issue close date (I see a difference of seconds), we look for
        # any pulls/issues closed a minute before the last release. User can
        # manually remove problematic things.
        since_datetime = dateutil.parser.parse(since)
        delta = datetime.timedelta(0, 60)
        since = (since_datetime - delta).isoformat()
        params = {'state': 'closed', 'since': since}
        recent_pulls = self.api_get("pulls", params=params).json()
        # print([p['number'] for p in recent_pulls], len(recent_pulls))
        # remove closed (unmerged) pulls
        recent_pulls = [p for p in recent_pulls if p['merged_at'] is not None]
        # print([p['number'] for p in recent_pulls], len(recent_pulls))
        # print([(p['number'], p['merged_at']) for p in recent_pulls])
        recent_pulls = self.filter_recent_pulls(recent_pulls, since)
        issue_params = {'filter': 'all'}
        issue_params.update(params)
        recent_issues = self.api_get_json_all("issues", params=issue_params)
        # print([(iss['number'], iss['closed_at']) for iss in recent_issues])
        issues_by_number = {iss['number']: iss for iss in recent_issues}
        # print(list(issues_by_number.keys()))
        desired_pulls = collections.defaultdict(list)
        for pull in recent_pulls:
            issue = issues_by_number[pull['number']]
            label_names = [label['name'] for label in issue['labels']]
            label_names = [None] if not label_names else label_names
            for label in label_names:
                desired_pulls[label] += [pull]

        return desired_pulls

    def _latest_release_tag_info(self, tag_name):
        tags = self.api_get("tags").json()
        desired_tag_list = [t for t in self.api_get("tags").json()
                            if t['name'] == tag_name]
        assert len(desired_tag_list) == 1
        self._latest_release_tag = desired_tag_list[0]

    @property
    def latest_release_commit_date(self):
        return self.set_release_info()

    def release_commit_date(self, release_name=None):
        # note that we use the date of the commit of the release, not the
        # date of the release itself (can release long after the commit)
        if release_name is None:
            release_name = "latest"
        else:
            release_name = "tags/" + release_name

        latest_release =  self.api_get("releases/" + release_name).json()
        if self._latest_release_tag_name != latest_release['tag_name']:
            self._latest_release_tag_info(latest_release['tag_name'])
            latest_tag_commit_sha = \
                self._latest_release_tag['commit']['sha']
            commit = self.api_get("commits/" + latest_tag_commit_sha).json()
            self._latest_release_commit_date = \
                     commit['commit']['committer']['date']
            self._latest_release_tag_name = latest_release['tag_name']
        return self._latest_release_commit_date

    def write_pull_line(self, pull, extra_labels=None):
        if extra_labels is None:
            extra_labels = []
        title = pull['title']
        number = str(pull['number'])
        author = pull['user']['login']
        url = pull['html_url']
        pull_link = "([#" + number + "](" + url + ")"
        if author not in self.config['standard_contributors']:
            pull_link += " @" + author
        pull_link += ")"
        out_str = "* " + title + " " + pull_link
        for label in extra_labels:
            label = label.replace(' ', '_')
            out_str += " #" + label
        out_str += "\n"
        return out_str

    @staticmethod
    def _pull_to_labels(pull_dict):
        pull_to_labels = collections.defaultdict(list)
        for label in pull_dict:
            for pull in pull_dict[label]:
                pull_to_labels[pull['number']] += [label]
        return pull_to_labels

    def output_for_known_labels(self, pull_dict, pull_to_labels):
        out_str = ""
        for lbl in self.config['labels']:
            label = lbl['label']
            out_str += "\n## " + lbl['heading'] + "\n"
            for pull in pull_dict[label]:
                pull_labels = set(pull_to_labels[pull['number']])
                extra_labels = pull_labels - set([label])
                out_str += self.write_pull_line(pull, extra_labels)
        return out_str

    def output_for_unknown_labels(self, pull_dict, unknown_labels,
                                  treated_pulls):
        out_str = "\n-----\n\n# Pulls with unknown labels\n"
        for label in unknown_labels:
            untreated = [pull for pull in pull_dict[label]
                         if pull['number'] not in treated_pulls]
            if untreated:
                out_str += "\n## " + str(label) + "\n"
            for pull in untreated:
                out_str += self.write_pull_line(pull)
                treated_pulls.append(pull['number'])
        return out_str

    def release_notes_from_pulls(self, pull_dict):
        config_labels = [lbl['label'] for lbl in self.config['labels']]
        unknown_labels = set(pull_dict) - set(config_labels)
        pull_to_labels = self._pull_to_labels(pull_dict)

        config_label_pulls = sum([pull_dict[lbl]
                                  for lbl in config_labels], [])
        treated_pulls = set([p['number'] for p in config_label_pulls])

        out_str = self.output_for_known_labels(pull_dict, pull_to_labels)

        if len(treated_pulls) != len(pull_to_labels):
            out_str += self.output_for_unknown_labels(pull_dict,
                                                      unknown_labels,
                                                      list(treated_pulls))
        return out_str

    def write_release_notes(self, outfile=None):
        if outfile is None:
            outfile = sys.stdout
        elif isinstance(outfile, str):
            outfile = open(outfile)
        release_date = self.release_commit_date(self.since_release)
        pull_dict = self.label_organized_merged_pulls(since=release_date)
        notes = self.release_notes_from_pulls(pull_dict)
        outfile.write(notes)
        outfile.flush()
        if outfile != sys.stdout:
            outfile.close()




import argparse
import time

try:
    from configparser import ConfigParser, NoSectionError, NoOptionError
except ImportError:
    # py2
    from ConfigParser import ConfigParser, NoSectionError, NoOptionError


from json import JSONDecodeError

from packaging.version import Version
import requests

def get_latest_pypi(package, index="https://test.pypi.org/pypi"):
    url = "/".join([index, package, 'json'])
    req = requests.get(url)
    try:
        version = max([Version(v) for v in req.json()['releases'].keys()])
    except JSONDecodeError:
        # couldn't find a version, so we're okay
        version = "0.0.0.dev0"
    return str(version)

def _strip_dev(version_str):
    version = Version(version_str)  # to normalize
    return str(version).split('.dev')[0]

def bump_dev_version(version_str):
    version = Version(version_str)
    if version.is_devrelease:
        dev_str = ".dev" + str(version.dev + 1)
        v_base, _ = str(version).split('.dev')
        out_v = v_base + dev_str
        pass
    else:
        out_v = version.public + ".dev0"
    return out_v

def select_version(v_pypi, v_setup):
    vv_pypi = _strip_dev(v_pypi)
    vv_setup = _strip_dev(v_setup)
    if Version(vv_setup) > Version(vv_pypi):
        return v_setup
    elif Version(vv_setup) == Version(vv_pypi):
        return v_pypi
    else:
        raise RuntimeError("Why does pypi have a newer version than setup?")

def shared_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--index", type=str,
                        default="https://test.pypi.org/pypi",
                        help="pypi index to search for versions")
    parser.add_argument("-c", "--conf", type=str, default="setup.cfg",
                        help="setup.cfg file to use")
    return parser
    parser.add_argument('--get-max', action='store_true')
    return parser

def get_version_info(conf_name, index):
    conf = ConfigParser()
    conf.read(conf_name)
    v_setup = conf.get('metadata', 'version')
    package = conf.get('metadata', 'name')
    v_pypi = get_latest_pypi(package, index)
    return conf, package, v_setup, v_pypi

def wait_for_max():
    parser = shared_parser()
    opts = parser.parse_args()

    found_desired = False
    while not found_desired:
        _, package, v_setup, v_pypi = get_version_info(opts.conf,
                                                       opts.index)
        print("Looking for version with base : %s" % v_setup)
        print("Maximum version on index: %s" % v_pypi)
        vv_setup = Version(v_setup)
        vv_pypi = Version(v_pypi)
        if vv_setup.base_version > vv_pypi.base_version:
            print("Waiting 5 seconds to see if package registers")
            time.sleep(5)
        elif vv_setup.base_version < vv_pypi.base_version:
            raise RuntimeError("Why is this version less than the index?")
        else:  # we are equal
            found_desired = True


def get_max():
    parser = argparse.ArgumentParser()
    parser.add_argument("package", type=str)
    parser.add_argument("-i", "--index", type=str,
                        default="https://test.pypi.org/pypi",
                        help="pypi index to search for versions")
    opts = parser.parse_args()
    print(get_latest_pypi(opts.package, opts.index))


def main():
    parser = shared_parser()
    parser.add_argument("-o", "--output", type=str,
                        help="output file; defaults to same as input")
    parser.add_argument('--dry', action='store_true')
    opts = parser.parse_args()
    output = opts.conf if opts.output is None else opts.output

    conf, package, v_setup, v_pypi = get_version_info(opts.conf, opts.index)
    version_to_bump = select_version(v_pypi, v_setup)
    new_version = bump_dev_version(version_to_bump)

    print("Local version: ", v_setup)
    print("Remote version: ", v_pypi)
    print("Bumped version: ", new_version)

    if not opts.dry:
        conf.set('metadata', 'version', new_version)
        with open(output, 'w') as f:
            conf.write(f)

if __name__ == "__main__":
    main()



# This file is vendored from Autorelease
import os
import ast
import sys

import fnmatch  # Py 2

from setuptools import setup

def _glob_glob_recursive(directory, pattern):
    # python 2 glob.glob doesn't have a recursive keyword
    # this implements for the specific case that we want an exact match
    # See also https://stackoverflow.com/a/2186565
    matches = []
    for root, dirname, filenames in os.walk(directory):
        matches.extend([os.path.join(root, filename)
                        for filename in fnmatch.filter(filenames, pattern)])
    return matches


class VersionPyFinder(object):
    _VERSION_PY_FUNCTIONS = ['get_git_version', 'get_setup_cfg']
    def __init__(self, filename='version.py', max_depth=2):
        self.filename_base = filename
        self.max_depth = max_depth
        self.depth = None
        self.filename = os.getenv("AUTORELEASE_VERSION_PY",
                                  self._first_eligible())
        self.functions = self._get_functions(self.filename)

    def _find_files(self):
        # all_files = glob.glob("**/" + self.filename_base, recursive=True)
        all_files = _glob_glob_recursive('.', self.filename_base)
        meets_depth = [fname for fname in all_files
                       if len(fname.split(os.sep)) <= self.max_depth + 1]
        return meets_depth

    def _is_eligible(self, filename):
        with open(filename, mode='r') as f:
            contents = f.read()

        tree = ast.parse(contents)
        # we requrie that our functions be defined at module level -- we
        # know that's how we wrote them, at least!
        all_functions = [node.name for node in tree.body
                         if isinstance(node, ast.FunctionDef)]
        return all(func in all_functions
                   for func in self._VERSION_PY_FUNCTIONS)

    def _first_eligible(self):
        all_files = self._find_files()
        for fname in all_files:
            if self._is_eligible(fname):
                return fname
        return None

    @property
    def version_setup_depth(self):
        def get_depth(fname):
            return len(os.path.abspath(fname).split(os.sep))

        # we assume thta setup.py is in the same dir as setup.cfg
        diff = get_depth(self.filename) - get_depth(__file__)
        return diff

    def _get_functions(self, filename):
        with open(self.filename, mode='r') as f:
            contents = f.read()

        tree = ast.parse(contents)

        class MakeImportError(ast.NodeTransformer):
            """converts a from x import y into an import error"""
            def __init__(self, import_name):
                self.import_name = import_name

            def visit_ImportFrom(self, node):
                if node.module == self.import_name:
                    replacement = ast.parse("raise ImportError()").body[0]
                    return ast.copy_location(replacement, node)
                else:
                    return node

        import_remover = MakeImportError("_installed_version")
        tree = import_remover.visit(tree)
        ast.fix_missing_locations(tree)

        locs = dict(globals())
        exec(compile(tree, filename="version.py", mode='exec'), locs)
        return {f: locs[f] for f in self._VERSION_PY_FUNCTIONS}


def write_installed_version_py(filename="_installed_version.py",
                               src_dir=None):
    version_finder = VersionPyFinder()
    directory = os.path.dirname(version_finder.filename)
    depth = version_finder.version_setup_depth
    get_git_version = version_finder.functions['get_git_version']
    get_setup_cfg = version_finder.functions['get_setup_cfg']

    installed_version = os.path.join(directory, "_installed_version.py")
    content = "_installed_version = '{vers}'\n"
    content += "_installed_git_hash = '{git}'\n"
    content += "_version_setup_depth = {depth}\n"

    # question: if I use the __file__ attribute in something I compile from
    # here, what is the file?
    my_dir = os.path.abspath(os.path.dirname(__file__))
    conf = get_setup_cfg(directory=my_dir, filename='setup.cfg')
    # conf = get_setup_cfg(directory=my_dir, filename='new_setup.cfg')
    version = conf.get('metadata', 'version')
    git_rev = get_git_version()

    # TODO: shouldn't vwe just use the directory found by the
    # VersionPyFinder?
    if src_dir is None:
        src_dir = conf.get('metadata', 'name')

    with open (os.path.join(src_dir, filename), 'w') as f:
        f.write(content.format(vers=version, git=git_rev, depth=depth))

if __name__ == "__main__":
    # TODO: only write version.py under special circumstances
    write_installed_version_py()
    # write_version_py(os.path.join('autorelease', 'version.py'))
    setup()




import git  # GitPython
import packaging.version as vers

def skipped_version(old_version, new_version, allow_patch_skip=False):
    old_major, old_minor, old_patch = old_version
    new_major, new_minor, new_patch = new_version

    bad_update = False

    if new_major == old_major:
        if new_minor == old_minor:
            if not allow_patch_skip and new_patch != old_patch + 1:
                bad_update = True

        elif new_patch != 0 and new_minor != old_minor + 1:
            bad_update = True

    elif new_minor != 0 != new_patch and new_major != old_major + 1:
        bad_update = True

    return bad_update


class GitReleaseChecks(object):
    def __init__(self, repo_path='.'):
        self.repo_path = repo_path
        self.repo = git.Repo(self.repo_path)
        self.repo.remotes.origin.fetch()
        self.stable_branch = 'stable'
        self.dev_branch = 'master'

    @staticmethod
    def tag_from_version(version):
        return "v" + str(version)

    def in_required_branch(self, required_branch='stable'):
        msg = ""
        current_branch = self.repo.active_branch.name
        if current_branch != required_branch:
            msg = ("Current branch is '" + str(current_branch)
                   + "'; should be in '" + str(required_branch) + "'\n")
        return msg

    def _versions_from_tags(self):
        # assume tags are format vM.m.p_build
        # this takes the version (before the '_') and drops the preceding v
        tag_versions = [t.name.split('_')[0][1:] for t in self.repo.tags]
        # TODO: may be better to replace with regex for reusability
        versions = []
        for version in tag_versions:
            try:
                v = vers.Version(version)
            except vers.InvalidVersion:
                pass
            else:
                versions.append(v)
        return versions
        # return [vers.Version(v) for v in tag_versions]

    def reasonable_desired_version(self, desired_version, allow_equal=False,
                                  allow_patch_skip=False):
        """
        Determine whether the desired version is a reasonable next version.

        Parameters
        ----------
        desired_version: str
            the proposed next version name
        """
        try:
            desired_version = desired_version.base_version
        except:
            pass
        (new_major, new_minor, new_patch) = \
                map(int, desired_version.split('.'))

        tag_versions = self._versions_from_tags()
        if not tag_versions:
            # no tags yet, and legal version is legal!
            return ""
        max_version = max(self._versions_from_tags()).base_version
        parts = list(map(int, str(max_version).split('.')))
        parts += [0] * (3 - len(parts))
        old_major, old_minor, old_patch = parts

        update_str = str(max_version) + " -> " + str(desired_version)

        v_desired = vers.Version(desired_version)
        v_max = vers.Version(max_version)

        no_increase_str = ("Bad update: New version doesn't increase on "
                           "last tag: " + update_str + "\n")

        if v_desired == v_max:
            if allow_equal:
                return ""
            else:
                return no_increase_str

        if v_desired < v_max:
            return no_increase_str

        bad_update = skipped_version((old_major, old_minor, old_patch),
                                     (new_major, new_minor, new_patch),
                                     allow_patch_skip)

        msg = ""
        if bad_update:
            msg = ("Bad update: Did you skip a version from "
                   + update_str + "?\n")

        return msg



from __future__ import print_function
import sys
import os
import textwrap
import argparse
import packaging.version as vers

import autorelease

class CheckRunner(object):
    def __init__(self, output):
        self.output = output or sys.stdout
        self.tests = []
        self.wrapper = textwrap.TextWrapper(width=78, initial_indent=" "*4,
                                            subsequent_indent=" "*4)

    def __call__(self, method, *args, **kwargs):
        """Generic method to turn other methods into tests.
        """
        self.output.write(str(method.__name__) + "... ")
        msg = method(*args, **kwargs)
        fail = 0
        if bool(msg):
            fail = 1
            self.output.write("FAIL\n")
            self.output.write(self.wrapper.fill(msg) + '\n')
        else:
            self.output.write("PASS\n")
        return fail

    def run(self, tests=None):
        tests = tests or self.tests  # take first non-None
        n_fails = 0
        for (method, args, kwargs) in tests:
            n_fails += self(method, *args, **kwargs)
        return n_fails

    def run_as_test(self, tests=None):
        n_fails = self.run(tests)
        if n_fails > 0:
            exit(1)


class DefaultCheckRunner(CheckRunner):
    def __init__(self, versions, setup, repo_path='.', output=None):
        self.version_checks = autorelease.VersionReleaseChecks(
            versions=versions,
            strictness='strict'
        )
        self.setup = setup
        self.git_repo_checks = autorelease.GitReleaseChecks(
            repo_path=repo_path
        )
        version_0 = list(versions.values())[0]
        self.desired_version = vers.Version(version_0)
        super(DefaultCheckRunner, self).__init__(output=output)
        self.consistency_test = (
            self.version_checks.consistency, [],
            {'include_package': False}
        )
        self.tests = [self.consistency_test]
        self.release_branches = ['stable']
        self.tag_branch = \
            self.git_repo_checks.tag_from_version(self.desired_version)


    def _is_release_tests(self, expected):
        return [
            (self.version_checks.is_release, [],
             {'version': self.desired_version,
              'expected': expected}),
            (autorelease.setup_is_release, [],
             {'setup': self.setup,
              'expected': expected})
        ]

    def _reasonable_desired_version_test(self, allow_equal,
                                         allow_patch_skip=False):
        print("allow equal ", allow_equal)
        print("allow patch skip", allow_patch_skip)
        return [
            (
                self.git_repo_checks.reasonable_desired_version, [],
                {'desired_version': self.desired_version,
                 'allow_equal': allow_equal,
                 'allow_patch_skip': allow_patch_skip}
            )
        ]

    @staticmethod
    def _get_branch_name(branch_name):
        if branch_name.startswith('refs/heads/'):
            branch = branch_name[11:]
        elif branch_name.startswith('refs/tags/'):
            branch = branch_name[10:]
        else:
            branch = branch_name

        return branch

    def select_tests(self):
        print(os.environ.get("GITHUB_ACTION"))
        if os.environ.get("GITHUB_ACTION", None):
            tests = self.select_test_from_github_env()
        else:
            tests = self.select_tests_from_sysargs()
        return tests


    def select_tests_from_sysargs(self):
        # TODO: this can be cleaned up by separating reusable parts
        parser = argparse.ArgumentParser()
        parser.add_argument('--branch', type=str)
        parser.add_argument('--event', type=str)
        parser.add_argument('--allow-patch-skip', action='store_true',
                            default=False)
        opts = parser.parse_args()

        branch = self._get_branch_name(opts.branch)
        return self.select_tests_from_branch_event(branch, opts.event,
                                                   opts.allow_patch_skip)

    def get_branch_event_from_github_env(self):
        event = os.environ.get("GITHUB_EVENT_NAME", None)
        ref = os.environ.get("GITHUB_REF", None)
        pr_ref = os.environ.get("GITHUB_BASE_REF", None)
        print(ref, pr_ref)
        print(os.environ.get("GITHUB_HEAD_REF", None))
        if event == "pull_request" and pr_ref is not None:
            branch = pr_ref
        elif event != "pull_request":
            branch = ref
        else:
            raise RuntimeError("PR without branch?")
        branch = self._get_branch_name(branch)
        return branch, event

    def select_test_from_github_env(self):
        parser = argparse.ArgumentParser()
        parser.add_argument('--allow-patch-skip', action='store_true',
                            default=False)
        opts = parser.parse_args()
        branch, event = self.get_branch_event_from_github_env()
        print(branch, event)
        return self.select_tests_from_branch_event(branch, event,
                                                   opts.allow_patch_skip)


    def select_tests_from_branch_event(self, branch, event, allow_patch_skip):
        if branch in self.release_branches:
            print("TESTING AS RELEASE")
            allow_equal = (event == 'cron'
                           or event == 'schedule'
                           or branch == self.tag_branch)
            tests = (self.tests
                     + self._reasonable_desired_version_test(
                         allow_equal=allow_equal,
                         allow_patch_skip=allow_patch_skip)
                     + self._is_release_tests(expected=True))
        else:
            print("TESTING AS NONRELEASE")
            tests = (self.tests
                     + self._reasonable_desired_version_test(False)
                     + self._is_release_tests(expected=False))

        return tests
